EpicEditor - An embeddable JavaScript Markdown editor
Description: EpicEditor is an embeddable JavaScript Markdown editor with split fullscreen editing, live previewing, automatic draft saving, offline support, and more.

#

## An Embeddable JavaScript Markdown Editor

EpicEditor is an embeddable JavaScript Markdown editor with split fullscreen editing, live previewing, automatic draft saving, offline support, and more. For developers, it offers a robust API, can be easily themed, and allows you to swap out the bundled Markdown parser with anything you throw at it.

## Why

Because, WYSIWYGs suck. Markdown is quickly becoming the replacement. GitHub, Stackoverflow, and even blogging apps like Posterous are now supporting Markdown. EpicEditor allows you to create a Markdown editor with a single line of JavaScript:

    var editor = new EpicEditor().load();

## Quick Start

EpicEditor is easy to implement. Add the script and assets to your page, provide a target container and call `load()`.

### Step 1: Download

Download the latest release or clone the repo:

    $ git clone git@github.com:OscarGodson/EpicEditor

### Step 2: Create your container element

    <div id="epiceditor"></div>

### Step 3: Add the `epiceditor.js` file

    <script src="epiceditor.min.js"></script>

### Step 4: Init EpicEditor

    var editor = new EpicEditor().load();

## API

### EpicEditor([options])

The `EpicEditor` constructor creates a new editor instance. Customize the instance by passing the `options` parameter. The example below uses all options and their defaults:

    var opts = {
      container: 'epiceditor',
      textarea: null,
      basePath: 'epiceditor',
      clientSideStorage: true,
      localStorageName: 'epiceditor',
      useNativeFullsreen: true,
      parser: marked,
      file: {
        name: 'epiceditor',
        defaultContent: '',
        autoSave: 100
      },
      theme: {
        base:'/themes/base/epiceditor.css',
        preview:'/themes/preview/preview-dark.css',
        editor:'/themes/editor/epic-dark.css'
      },
      focusOnLoad: false,
      shortcut: {
        modifier: 18,
        fullscreen: 70,
        preview: 80
      },
      string: {
        togglePreview: 'Toggle Preview Mode',
        toggleEdit: 'Toggle Edit Mode',
        toggleFullscreen: 'Enter Fullscreen'
      }
    }
    var editor = new EpicEditor(opts);

### Options

Option | Description | Default
--- | --- | ---
`container` | The ID (string) or element (object) of the target container in which you want the editor to appear. | `epiceditor`
`textarea` | The ID (string) or element (object) of a textarea you would like to sync the editor's content with. |
`basePath` | The base path of the directory containing the `/themes`, `/images`, etc. | `epiceditor`
`clientSideStorage` | Setting this to false will disable localStorage. | `true`
`localStorageName` | The name to use for the localStorage object. | `epiceditor`
`useNativeFullscreen` | Set to false to always use faux fullscreen (the same as what is used for unsupported browsers). | `true`
`parser` | Marked is the only parser built into EpicEditor, but you can customize or toggle this by passing a parsing function to this option. For example: `parser: MyCustomParser.parse` | `marked`
`focusOnLoad` | If `true`, editor will focus on load. | `false`
`file.name` | If no file exists with this name a new one will be made, otherwise the existing will be opened. | container ID
`file.defaultContent` | The content to show if no content exists for a file. |
`file.autoSave` | How often to auto save the file in milliseconds. Set to `false` to turn it off. | `100`
`theme.base` | The base styles such as the utility bar with the buttons. | `themes/base/epiceditor.css`
`theme.editor` | The theme for the editor which is the area you type into. | `themes/editor/epic-dark.css`
`theme.preview` | The theme for the previewer. | `themes/preview/github.css`
`shortcut.modifier` | The key to hold while holding the other shortcut keys to trigger a key combo. | `18` (`alt` key)
`shortcut.fullscreen` | The shortcut to open fullscreen. | `70` (`f` key)
`shortcut.preview` | The shortcut to toggle the previewer. | `80` (`p` key)
`string.togglePreview` | The tooltip text that appears when hovering the preview icon. | `Toggle Preview Mode`
`string.toggleEdit` | The tooltip text that appears when hovering the edit icon. | `Toggle Edit Mode`
`string.toggleFullscreen` | The tooltip text that appears when hovering the fullscreen icon. | `Enter Fullscreen`

### load([callback])

Loads the editor by inserting it into the DOM by creating an `iframe`. Will trigger the `load` event, or you can provide a callback.

    editor.load(function () {
      console.log("Editor loaded.")
    });

### unload([callback])

Unloads the editor by removing the `iframe`. Keeps any options and file contents so you can easily call `.load()` again. Will trigger the `unload` event, or you can provide a callback.

    editor.unload(function () {
      console.log("Editor unloaded.")
    });

### getElement(element)

Grabs an editor element for easy DOM manipulation. See the Themes section below for more on the layout of EpicEditor elements.

- `container`: The element given at setup in the options.
- `wrapper`: The wrapping `<div>` containing the 2 editor and previewer iframes.
- `wrapperIframe`: The iframe containing the `wrapper` element.
- `editor`: The #document of the editor iframe (i.e. you could do `editor.getElement('editor').body`).
- `editorIframe`: The iframe containing the `editor` element.
- `previewer`: The #document of the previewer iframe (i.e. you could do `editor.getElement('previewer').body`).
- `previewerIframe`: The iframe containing the `previewer` element.

    someBtn.onclick = function () {
      console.log(editor.getElement('editor').body.innerHTML); // Returns the editor's content
    }

### is(state)

Returns a boolean for the requested state. Useful when you need to know if the editor is loaded yet for example. Below is a list of supported states:

- `loaded`
- `unloaded`
- `edit`
- `preview`
- `fullscreen`

    fullscreenBtn.onclick = function () {
      if (!editor.is('loaded')) { return; }
      editor.enterFullscreen();
    }

### open(filename)

Opens a file into the editor.

    openFileBtn.onclick = function () {
      editor.open('some-file'); // Opens a file when the user clicks this button
    }

### importFile([filename],[content])

Imports a string of content into a file. If the file already exists, it will be overwritten. Useful if you want to inject a bunch of content via AJAX. Will also run `.open()` after import automatically.

    importFileBtn.onclick = function () {
      editor.importFile('some-file',"#Imported markdown\nFancy, huh?"); //Imports a file when the user clicks this button
    }

### exportFile([filename],[type])

Returns the raw content of the file by default, or if given a `type` will return the content converted into that type. If you leave both parameters `null` it will return the current document's raw content.

    syncWithServerBtn.onclick = function () {
      var theContent = editor.exportFile();
      saveToServerAjaxCall('/save', {data:theContent}, function () {
        console.log('Data was saved to the database.');
      });
    }

### rename(oldName, newName)

Renames a file.

    renameFileBtn.onclick = function () {
      var newName = prompt('What do you want to rename this file to?');
      editor.rename('old-filename.md', newName); //Prompts a user and renames a file on button click
    }

### save()

Manually saves a file. EpicEditor will save continuously every 100ms by default, but if you set `autoSave` in the options to `false` or to longer intervals it's useful to manually save.

    saveFileBtn.onclick = function () {
      editor.save();
    }

### remove(name)

Deletes a file.

    removeFileBtn.onclick = function () {
      editor.remove('example.md');
    }

### getFiles([name])

If no `name` is given it returns an object containing all the file objects. If a `name` is specified it will return just that single file object.

    var files = editor.getFiles();
    for (x in files) {
      console.log('File: ' + x); //Returns the name of each file
    };

### on(event, handler)

Sets up an event handler (callback) for a specified event. For all event types, see the Events section below.

    editor.on('unload', function () {
      console.log('Editor was removed');
    });

### emit(event)

Fires an event programatically. Similar to jQuery's `.trigger()`

    editor.emit('unload'); // Triggers the handler provided in the "on" method above

### removeListener(event, [handler])

Allows you to remove all listeners for an event, or just the specified one.

    editor.removeListener('unload'); //The handler above would no longer fire

### preview()

Puts the editor into preview mode.

    previewBtn.onclick = function () {
      editor.preview();
    }

### edit()

Puts the editor into edit mode.

    editBtn.onclick = function () {
      editor.edit();
    }

### enterFullscreen()

Puts the editor into fullscreen mode.

Note: due to browser security restrictions, calling `enterFullscreen` programmatically like this will not trigger native fullscreen. Native fullscreen can only be triggered by a user interaction like mousedown or keyup.

    enterFullscreenBtn.onclick = function () {
      editor.enterFullscreen();
    }

### exitFullscreen()

Closes fullscreen mode.

    exitFullscreenBtn.onclick = function () {
      editor.exitFullscreen();
    }

### reflow([type], [callback])

`reflow()` allows you to "reflow" the editor in it's container. For example, let's say you increased the height of your wrapping element and want the editor to resize too. You could call `reflow` and the editor will resize to fit. You can pass it one of two strings as the first parameter to constrain the reflow to either `width` or `height`.

It also provides you with a callback parameter if you'd like to do something after the resize is finished. The callback will return the new width and/or height in an object. Additionally, you can also listen for the `reflow` event. This will also give you back the new size.

Note: If you call `reflow()` or `reflow('width')` and you have a fluid width container EpicEditor will no longer be fluid because doing a reflow on the width sets an inline style on the editor.

    // For an editor that takes up the whole browser window:
    window.onresize = function () {
      editor.reflow();
    }

    // Constrain the reflow to just height:
    someDiv.resizeHeightHandle = function () {
      editor.reflow('height');
    }

    // Same as the first example, but this has a callback
    window.onresize = function () {
      editor.reflow(function (data) {
        console.log('width: ', data.width, ' ', 'height: ', data.height);
      });
    }

## Events

You can hook into specific events in EpicEditor with `on()` such as when a file is created, removed, or updated. Below is a complete list of currently supported events and their description.

Event Name | Description
--- | ---
`create` | Fires whenever a new file is created.
`read` | Fires whenever a file is read.
`update` | Fires whenever a file is updated.
`remove` | Fires whenever a file is deleted.
`load` | Fires when the editor loads via `load()`.
`unload` | Fires whenever the editor is unloaded via `unload()`
`preview` | Fires whenever the previewer is opened (excluding fullscreen) via `preview()` or the preview button.
`edit` | Fires whenever the editor is opened (excluding fullscreen) via `edit()` or the edit button.
`fullscreenenter` | Fires whenever the editor opens in fullscreen via `fullscreen()` or the fullscreen button.
`fullscreenexit` | Fires whenever the editor closes in fullscreen via `fullscreen()` or the fullscreen button.
`save` | Fires whenever the file is saved whether by EpicEditor automatically or when `save()` is called.
`open` | Fires whenever a file is opened or loads automatically by EpicEditor or when `open()` is called.
`reflow` | Fires whenever `reflow()` is called. Will return the new dimensions in the callback.

## Themes

Theming is easy in EpicEditor. There are three different `<iframe>`s which means styles wont leak between the "chrome" of EpicEditor, previewer, or editor. Each one is like it's own web page. In the `themes` directory you'll see `base`, `preview`, and `editor`. The base styles are for the "chrome" of the editor which contains elements such as the utility bar containing the icons. The editor is the styles for the contents of editor `<iframe>` and the preview styles are applied to the preview `<iframe>`.

The HTML of a generated editor (excluding contents) looks like this:

    <div id="container">
      <iframe id="epiceditor-instance-id">
        <html>
          <head>
            <link type="text/css" id="" rel="stylesheet" href="epiceditor/themes/base/epiceditor.css" media="screen">
          </head>
          <body>
            <div id="epiceditor-wrapper">
              <iframe id="epiceditor-editor-frame">
                <html>
                  <head>
                    <link type="text/css" rel="stylesheet" href="epiceditor/themes/editor/epic-dark.css" media="screen">
                  </head>
                  <body contenteditable="true">
                    <!-- raw content -->
                  </body>
                </html>
              </iframe>
              <iframe id="epiceditor-previewer-frame">
                <html>
                  <head>
                    <link type="text/css" rel="stylesheet" href="epiceditor/themes/preview/github.css" media="screen">
                  </head>
                  <body>
                    <div id="epiceditor-preview">
                      <!-- rendered html -->
                    </div>
                  </body>
                </html>
              </iframe>
              <div id="epiceditor-utilbar">
                <img width="16" src="epiceditor/images/preview.png" class="epiceditor-toggle-btn">
                <img width="16" src="epiceditor/images/fullscreen.png" class="epiceditor-fullscreen-btn">
              </div>
            </div>
          </body>
        </html>
      </iframe>
    </div>

## Custom Parsers

EpicEditor is set up to allow you to use any parser that accepts and returns a string. This means you can use any flavor of Markdown, process Textile, or even create a simple HTML editor/previewer (`parser: false`). The possibilities are endless. Just make the parser available and pass its parsing function to the EpicEditor setting and you should be all set.

For even more customization/optimization you can replace the default built-in processor on build. Running `jake build parser=path/to/parser.js` will override the default Marked build and replace it with your custom script.

See the custom parsers wiki page for more.

## Contributing

Contributions are greatly encouraged and appreciated. For more on ways to contribute please check the wiki: Contributing Guide.

## Credits

EpicEditor relies on Marked to parse markdown and is brought to you in part by Oscar Godson and John Donahue. Special thanks to Adam Bickford for the bug fixes and being the QA for pull requests. Lastly, huge thanks to Sebastian Nitu for the amazing logo and doc styles.